package org.apache.bcel.classfile;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import org.apache.bcel.Const;
import org.apache.bcel.classfile.AnnotationDefault;
import org.apache.bcel.classfile.AttributeReader;
import org.apache.bcel.classfile.BootstrapMethods;
import org.apache.bcel.classfile.Code;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.ConstantValue;
import org.apache.bcel.classfile.Deprecated;
import org.apache.bcel.classfile.EnclosingMethod;
import org.apache.bcel.classfile.ExceptionTable;
import org.apache.bcel.classfile.InnerClasses;
import org.apache.bcel.classfile.LineNumberTable;
import org.apache.bcel.classfile.LocalVariableTable;
import org.apache.bcel.classfile.LocalVariableTypeTable;
import org.apache.bcel.classfile.MethodParameters;
import org.apache.bcel.classfile.Module;
import org.apache.bcel.classfile.ModuleMainClass;
import org.apache.bcel.classfile.ModulePackages;
import org.apache.bcel.classfile.NestHost;
import org.apache.bcel.classfile.NestMembers;
import org.apache.bcel.classfile.Node;
import org.apache.bcel.classfile.PMGClass;
import org.apache.bcel.classfile.RuntimeInvisibleAnnotations;
import org.apache.bcel.classfile.RuntimeInvisibleParameterAnnotations;
import org.apache.bcel.classfile.RuntimeVisibleAnnotations;
import org.apache.bcel.classfile.RuntimeVisibleParameterAnnotations;
import org.apache.bcel.classfile.Signature;
import org.apache.bcel.classfile.SourceFile;
import org.apache.bcel.classfile.StackMap;
import org.apache.bcel.classfile.Synthetic;
import org.apache.bcel.classfile.Unknown;
import org.apache.bcel.classfile.UnknownAttributeReader;
import org.apache.bcel.classfile.Visitor;

public abstract class Attribute implements Cloneable, Node {
  private static final boolean debug = Boolean.getBoolean(org.apache.bcel.classfile.Attribute.class.getCanonicalName() + ".debug");
  
  private static final Map<String, Object> READERS = new HashMap<>();
  
  public static final org.apache.bcel.classfile.Attribute[] EMPTY_ARRAY = new org.apache.bcel.classfile.Attribute[0];
  
  @Deprecated
  protected int name_index;
  
  @Deprecated
  protected int length;
  
  @Deprecated
  protected byte tag;
  
  @Deprecated
  protected ConstantPool constant_pool;
  
  @Deprecated
  public static void addAttributeReader(String name, AttributeReader attributeReader) {
    READERS.put(name, attributeReader);
  }
  
  public static void addAttributeReader(String name, UnknownAttributeReader unknownAttributeReader) {
    READERS.put(name, unknownAttributeReader);
  }
  
  protected static void println(String msg) {
    if (debug)
      System.err.println(msg); 
  }
  
  public static org.apache.bcel.classfile.Attribute readAttribute(DataInput file, ConstantPool constantPool) throws IOException {
    Object r;
    byte tag = -1;
    int nameIndex = file.readUnsignedShort();
    String name = constantPool.getConstantUtf8(nameIndex).getBytes();
    int length = file.readInt();
    byte i;
    for (i = 0; i < 27; i = (byte)(i + 1)) {
      if (name.equals(Const.getAttributeName(i))) {
        tag = i;
        break;
      } 
    } 
    switch (tag) {
      case -1:
        r = READERS.get(name);
        if (r instanceof UnknownAttributeReader)
          return ((UnknownAttributeReader)r).createAttribute(nameIndex, length, file, constantPool); 
        return new Unknown(nameIndex, length, file, constantPool);
      case 1:
        return new ConstantValue(nameIndex, length, file, constantPool);
      case 0:
        return new SourceFile(nameIndex, length, file, constantPool);
      case 2:
        return new Code(nameIndex, length, file, constantPool);
      case 3:
        return new ExceptionTable(nameIndex, length, file, constantPool);
      case 4:
        return new LineNumberTable(nameIndex, length, file, constantPool);
      case 5:
        return new LocalVariableTable(nameIndex, length, file, constantPool);
      case 6:
        return new InnerClasses(nameIndex, length, file, constantPool);
      case 7:
        return new Synthetic(nameIndex, length, file, constantPool);
      case 8:
        return new Deprecated(nameIndex, length, file, constantPool);
      case 9:
        return new PMGClass(nameIndex, length, file, constantPool);
      case 10:
        return new Signature(nameIndex, length, file, constantPool);
      case 11:
        println("Warning: Obsolete StackMap attribute ignored.");
        return new Unknown(nameIndex, length, file, constantPool);
      case 12:
        return new RuntimeVisibleAnnotations(nameIndex, length, file, constantPool);
      case 13:
        return new RuntimeInvisibleAnnotations(nameIndex, length, file, constantPool);
      case 14:
        return new RuntimeVisibleParameterAnnotations(nameIndex, length, file, constantPool);
      case 15:
        return new RuntimeInvisibleParameterAnnotations(nameIndex, length, file, constantPool);
      case 16:
        return new AnnotationDefault(nameIndex, length, file, constantPool);
      case 17:
        return new LocalVariableTypeTable(nameIndex, length, file, constantPool);
      case 18:
        return new EnclosingMethod(nameIndex, length, file, constantPool);
      case 19:
        return new StackMap(nameIndex, length, file, constantPool);
      case 20:
        return new BootstrapMethods(nameIndex, length, file, constantPool);
      case 21:
        return new MethodParameters(nameIndex, length, file, constantPool);
      case 22:
        return new Module(nameIndex, length, file, constantPool);
      case 23:
        return new ModulePackages(nameIndex, length, file, constantPool);
      case 24:
        return new ModuleMainClass(nameIndex, length, file, constantPool);
      case 25:
        return new NestHost(nameIndex, length, file, constantPool);
      case 26:
        return new NestMembers(nameIndex, length, file, constantPool);
    } 
    throw new IllegalStateException("Unrecognized attribute type tag parsed: " + tag);
  }
  
  public static org.apache.bcel.classfile.Attribute readAttribute(DataInputStream file, ConstantPool constantPool) throws IOException {
    return readAttribute(file, constantPool);
  }
  
  public static void removeAttributeReader(String name) {
    READERS.remove(name);
  }
  
  protected Attribute(byte tag, int nameIndex, int length, ConstantPool constantPool) {
    this.tag = tag;
    this.name_index = nameIndex;
    this.length = length;
    this.constant_pool = constantPool;
  }
  
  public abstract void accept(Visitor paramVisitor);
  
  public Object clone() {
    org.apache.bcel.classfile.Attribute attr = null;
    try {
      attr = (org.apache.bcel.classfile.Attribute)super.clone();
    } catch (CloneNotSupportedException e) {
      throw new Error("Clone Not Supported");
    } 
    return attr;
  }
  
  public abstract org.apache.bcel.classfile.Attribute copy(ConstantPool paramConstantPool);
  
  public void dump(DataOutputStream file) throws IOException {
    file.writeShort(this.name_index);
    file.writeInt(this.length);
  }
  
  public final ConstantPool getConstantPool() {
    return this.constant_pool;
  }
  
  public final int getLength() {
    return this.length;
  }
  
  public String getName() {
    return this.constant_pool.getConstantUtf8(this.name_index).getBytes();
  }
  
  public final int getNameIndex() {
    return this.name_index;
  }
  
  public final byte getTag() {
    return this.tag;
  }
  
  public final void setConstantPool(ConstantPool constantPool) {
    this.constant_pool = constantPool;
  }
  
  public final void setLength(int length) {
    this.length = length;
  }
  
  public final void setNameIndex(int nameIndex) {
    this.name_index = nameIndex;
  }
  
  public String toString() {
    return Const.getAttributeName(this.tag);
  }
}
